<!DOCTYPE html>
<html lang="en-US">
    <head>
        <meta charset="UTF-8">
        <title>BAD - Home</title>
    </head>
    <body>
        <h2>BAD - Benchmarking for Anomaly Detection</h2>
        <p>Hello from BAD worker!</p>
    </body>
</html>