"""Copyright (C) 2020 Sivam Pasupathipillai <s.pasupathipillai@unitn.it>.

All rights reserved.
"""
import os

BAD_DIR = ".bad"
BAD_CONF_DIR = os.path.join(BAD_DIR, "bad.conf")
LOG_FORMAT = "%(asctime)s - %(name)s - %(levelname)5s - %(message)s"
